;;;  -*- emacs-lisp -*-
;;;
;;;  $Id: irchat-copyright.el,v 3.2 1997/02/27 10:19:14 jsl Exp $
;;;
(defvar irchat-copyright "
    $Id: irchat-copyright.el,v 3.2 1997/02/27 10:19:14 jsl Exp $
    Internet Relay CHAT interface for GNU Emacs
    Copyright (C) 1989 Tor Lillqvist
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with GNU Emacs; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
    Author's email address is <tml@hemuli.atk.vtt.fi>
  
    Following people have made major improvements to irchat. Also there
    are numerous people who have given suggestions and improvements. If 
    you think your name should be included here, give us a note...
	   Kai Keinänen		<kmk@iki.fi>
	   Markku Järvinen	<mta@cc.tut.fi>
	   Jukka Partanen      	<jtp@iki.fi>
	   Kim Nyberg          	<kny@iki.fi>
	   Tero T. Mononen     	<tmo@iki.fi>

    You can ftp the latest version from cs.hut.fi:/pub/irchat

    There is a mailing list, irchat@cc.tut.fi for those interested
    in developments in irchat. Mail irchat-request@cc.tut.fi if you
    are interested in joining.

    Current team is:
	   Janne Liimatainen	<jsl@iki.fi>
	   Tero T. Mononen     	<tmo@iki.fi>
	   Kim Nyberg          	<kny@iki.fi>
           Tomi Ollila          <too@iki.fi>
	   Jukka Partanen      	<jtp@iki.fi>
	   Timo J. Rinne	<tri@iki.fi>
" "Basically GPL")

;;;
;;;  $Log: irchat-copyright.el,v $
;;;  Revision 3.2  1997/02/27 10:19:14  jsl
;;;  changed irchat-nickname to irchat-real-nickname
;;;  users should only change irchat-nickname
;;;
;;;Revision 3.1  1997/02/24  16:00:02  tri
;;;Promoted revision to 3.1.
;;;
;;;  Revision 1.2  1997/02/13 09:10:47  tri
;;;  Added Domo to "current" team.
;;;
;;;  Revision 1.1  1996/12/19 14:54:48  tri
;;;  Initial revision
;;;
;;;  Revision 0.1.1  1995/01/15  11:43:37  tmo
;;;  Initial revision
;;; 
;;;  Revision 0.1.2  1993/07/13  11:56:52  tmo
;;;  	made this file to be defvar
;;; 
;;;  Revision 0.1.2  1993/07/13  11:56:52  tmo
;;;  	made this file to be defvar
;;; 
;;;  Revision 0.1.1  1993/07/10  13:23:50  tmo
;;;  Initial revision
;;; 
